"""The product block that describes a site subscription."""
from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle, strEnum

from gso.schemas.types import LatitudeCoordinate, LongitudeCoordinate


class SiteTier(strEnum):
    """The tier of a site, ranging from 1 to 4.

    A lower value corresponds to a larger amount of installed equipment, and a higher cardinality of the connectivity to
    and from a site.
    """

    TIER1 = 1
    TIER2 = 2
    TIER3 = 3
    TIER4 = 4


class SiteBlockInactive(ProductBlockModel, lifecycle=[SubscriptionLifecycle.INITIAL], product_block_name="SiteBlock"):
    """A site that's currently inactive, see :class:`SiteBlock`."""

    site_name: str | None = None
    site_city: str | None = None
    site_country: str | None = None
    site_country_code: str | None = None
    site_latitude: LatitudeCoordinate | None = None
    site_longitude: LongitudeCoordinate | None = None
    site_internal_id: int | None = None
    site_bgp_community_id: int | None = None
    site_tier: SiteTier | None = None
    site_ts_address: str | None = None


class SiteBlockProvisioning(SiteBlockInactive, lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    """A site that's currently being provisioned, see :class:`SiteBlock`."""

    site_name: str | None = None
    site_city: str | None = None
    site_country: str | None = None
    site_country_code: str | None = None
    site_latitude: LatitudeCoordinate | None = None
    site_longitude: LongitudeCoordinate | None = None
    site_internal_id: int | None = None
    site_bgp_community_id: int | None = None
    site_tier: SiteTier | None = None
    site_ts_address: str | None = None


class SiteBlock(SiteBlockProvisioning, lifecycle=[SubscriptionLifecycle.ACTIVE]):
    """A site that's currently available for routers and services to be hosted at."""

    #:  The name of the site, that will dictate part of the :term:`FQDN` of routers that are hosted at this site. For
    #:  example: ``router.X.Y.geant.net``, where X denotes the name of the site.
    site_name: str
    #:  The city at which the site is located.
    site_city: str
    #:  The country in which the site is located.
    site_country: str
    #:  The code of the corresponding country. This is also used for the :term:`FQDN`, following the example given for
    #:  the site name, the country code would end up in the Y position.
    site_country_code: str
    #:  The latitude of the site, used for :term:`SNMP` purposes.
    site_latitude: LatitudeCoordinate
    #:  Similar to the latitude, the longitude of a site.
    site_longitude: LongitudeCoordinate
    #:  The internal ID used within GÉANT to denote a site.
    site_internal_id: int
    #:  The :term:`BGP` community ID of a site, used to advertise routes learned at this site.
    site_bgp_community_id: int
    #:  The tier of a site, as described in :class:`SiteTier`.
    site_tier: SiteTier
    #:  The address of the terminal server that this router is connected to. The terminal server provides out of band
    #:  access. This is required in case a link goes down, or when a router is initially added to the network and it
    #:  does not have any IP trunks connected to it yet.
    site_ts_address: str | None = None
